<?php

/**
 * @file
 * Provision/Drush hooks for the provision_drulenium module.
 *
 * These are the hooks that will be executed by the drush_invoke function.
 */

/**
 * Implementation of hook_drush_command().
 */
function provision_drulenium_drush_command() {
  $items['provision-dr_create'] = array(
    'description' => 'Create Drulenium screenshots',
    'bootstrap' => DRUSH_BOOTSTRAP_DRUSH,
  );
  $items['provision-dr_run_compare'] = array(
    'description' => 'Run compare for Drulenium',
    'options' => array(
      'keep-from' => 'Do not update screenshots for the reference site',
      'keep-to' => 'Do not update screenshots for the site under test',
    ),
    'arguments' => array(
      'from' => 'site alias of the reference site.',
      'to' => 'site alias of the site under test.',
    ),
  );

  return $items;
}


/**
 * Implements the provision-drulenium_dr_run_compare command.
 */
function drush_provision_drulenium_provision_dr_run_compare($alias_from = NULL, $alias_to = NULL) {
  if (d()->name == '@hostmaster') {
    hosting_drulenium_start_compare_task($alias_from, $alias_to);
  }
  else {
    // No OP, this runs in the context of $alias_from. And we want to work in @hostmaster.
    drush_log('Postponing processing until hosting_drulenium_post_hosting_dr_run_compare_task to get into the right context', 'debug');
  }
}

/**
 * Implements the provision-drulenium_dr_run_compare command.
 */
function drush_provision_drulenium_provision_dr_create() {
  // No OP, this runs in the subject site. And we want to work in @hostmaster.
}

/**
 * Implements hook_post_hosting_TASK_TYPE_task().
 */
function hosting_drulenium_post_hosting_dr_create_task($task) {
  //drush_log(__LINE__ . print_r($task, 1), 'info');
  $nid = $task->ref->nid;
  $uri = $task->ref->hosting_name;
  $rid = _drush_dr_create_release_id($nid, $uri);
}

/**
 * Implements hook_post_hosting_TASK_TYPE_task().
 */
function hosting_drulenium_post_hosting_dr_run_compare_task($task) {
  //drush_log(print_r($task, 1), 'info');
  $alias_from = $task->ref->title;
  $alias_to = $task->task_args['reference'];
  $keep_from = $task->task_args['keep-from'];
  $keep_to = $task->task_args['keep-to'];
  hosting_drulenium_start_compare_task($alias_from, $alias_to, $keep_from, $keep_to);
}

function hosting_drulenium_start_compare_task($alias_from = NULL, $alias_to = NULL, $keep_from = FALSE, $keep_to = FALSE) {
  drush_errors_on();
  drush_log("Comparing from:$alias_from - to:$alias_to", 'info');

  // Lookup if a release already exists for the aliases.
  $rid_from = _drush_alias_to_release_id($alias_from, $keep_from);
  $rid_to = _drush_alias_to_release_id($alias_to, $keep_to);

  // Cheat... assume the diff function can just take the last two create releases.
  //$rid_from = $rid_to = null;

  drush_drulenium_visual_regression_diff($rid_from, $rid_to, $reset = TRUE);
  // Alternative.
  //  provision_backend_invoke('@self', 'vr-diff', array($rid_from, $rid_to));

  global $base_url;

  drush_log("View results on $base_url/" . _drulenium_secure_compare_link($rid_from, $rid_to), 'success');

  // Tell hostmaster no changes need to be saved to it's drush context.
  drush_set_option('provision_save_config', false);
}

function _drush_alias_to_site_nid($alias) {

  $alias = ltrim($alias, '@');

  // Lookup the Aegir node ID for the site.
  $nodes = node_load_multiple(NULL, array("title" => $alias, 'type' => 'site'));
  $node = current($nodes);
  $nid = $node->nid;
  drush_log(dt('Found @nid as node ID for @name', array('@nid' => $nid, '@name' => $alias)), 'notice');
  return $nid;
}

/**
 * Translate a Drush alias name to a Drulenium relese ID.
 *
 * @param string $alias
 *   The Drush alias.
 *
 * @param bool $keep
 *   Option to keep exsisting screenshots, or create new ones.
 */
function _drush_alias_to_release_id($alias, $keep, $site_nid = NULL) {
  $alias = ltrim($alias, '@');

  if (empty($site_nid)) {
    $site_nid = _drush_alias_to_site_nid($alias);
  }

  if ($keep) {
    $query = db_select('drulenium_vr_releases', 'vr_re');
    $query->leftjoin('hosting_drulenium_vr_releases', 't', 'vr_re.rid = t.rid');
    $query->condition('t.nid', $site_nid)
      ->orderBy('vr_re.rid', 'DESC')
      ->range(0, 1);
    $query->fields('vr_re', array('rid'));
    $release_id = $query->execute()->fetchfield();
  }


  // Either not keeping the old screenshot, or none found.
  if (empty($release_id)) {
    $release_id = _drush_dr_create_release_id($site_nid, $alias);
  }
  else {
    drush_log("Keeping earlier snapshot of $alias", 'ok');
  }
  return $release_id;
}

function _drush_dr_create_release_id($site_nid, $uri) {
  $node = node_load($site_nid);
  if (empty($uri)) {
   $uri = $node->hosting_name;
  }

  // Prepare parameters.
  $auth = '';
  if (!empty($node->http_basic_auth_username)) {
    $auth = $node->http_basic_auth_username . ':' . $node->http_basic_auth_password . '@';
  }
  $scheme = 'http://';
  if (!empty($node->ssl_enabled) && $node->ssl_enabled > 0) {
    $scheme = 'https://';
  }
  $url = $scheme . $auth . $uri . '/';
  $name = $uri . '@' . format_date(REQUEST_TIME, 'short');

  // Do the creation.
  provision_backend_invoke('@hostmaster', 'vr-create', array($url, $name), array('pages' => implode(',', explode("\n", $node->hosting_drulenium_site_pages))));

  // Lookup the new release_id.
  $query = db_select('drulenium_vr_releases', 'vr_re');
  $release_id = $query
    ->fields('vr_re', array('rid'))
    ->condition('vr_re.name', $name)
    ->execute()->fetchfield();

  if (!empty($site_nid)) {
    // Store the relation.
    $record = array('rid' => $release_id, 'nid' => $site_nid);
    drupal_write_record('hosting_drulenium_vr_releases', $record);
  }

  return $release_id;
}
